/*******************

Windidi 
	by Fishuyo


 source code
 based on KAOS-DS
            by 0xtob

Source code licensed
under the GPL.
*******************/

#include <nds.h>
#include <nds/arm9/console.h>

#include <stdio.h>
#include <stdarg.h>

#include "command.h"
#include "libdsmi.h"

#include "windidi_logo.h"

#define PEN_DOWN (~IPC->buttons & (1 << 6))
//#define ABS(x) (((x) < 0) ? -(x) : (x))
#define POS(x) (((x) < 0) ? 0 : (x))
typedef struct colorBar{
	u8 r, g, b;
	u8 x, x0, h;
	s16 dx, bright;
};
 

colorBar channelBars [16];
colorBar noteBars [4];

touchPosition touch;
int touch_was_down = 0;
int is_playing = 0;
int mic_enabled = 0;

int noteindx = 0, micindx = 0;
u8 lastnote = 0, curnote = 0;
u8 notes[4] = {30,32,34,35};

u8 pm_x0 = 0, pm_y0 = 0, wind_mic;
u8 channel = 0;

int micavrg = 0, oldmicavrg = 0; 
u8 mic[8] = {0,0,0,0,0,0,0,0};
//u8 dmic[8] = {0,0,0,0,0,0,0,0};
//u8 davrg = 0, doldavrg = 0;  

u16 *sub_bg3vram = (u16*)BG_BMP_RAM_SUB(0);
u16 *bg3vram = (u16*)BG_BMP_RAM(8);
u16 *bg2vram = (u16*)BG_BMP_RAM(2);

void sgIP_dbgprint(char * txt, ...) {
        char buffer[256];
        va_list args;
        va_start(args,txt);
        vsprintf(buffer,txt,args);
        iprintf(buffer);
}


void play(u8 note, u8 vol){
	dsmi_write(0x90 | channel, note, vol);
}

void stop(u8 note){
	dsmi_write(0x80 | channel, note, 0);
}
void stopAll(){
	dsmi_write(0xB0 + channel, 0x7B, 0);
}
void newTouch(u8 x, u8 y)
{
	u8 scaled_x = x * 128 / 256;
	if(scaled_x > 127) scaled_x = 127;
	u8 scaled_y = (191 - y) * 128 / 192;
	if(scaled_y > 127) scaled_y = 127;
	stop(lastnote);
	curnote = scaled_x;
	is_playing = 0;
	channelBars[channel].x0 = channelBars[channel].x;
	channelBars[channel].x = x;
	channelBars[channel].dx = 0;
	if(!mic_enabled) {
		play(curnote, scaled_y);
		lastnote = curnote;
		channelBars[channel].h = 191 - y;
		channelBars[channel].bright = scaled_y >> 3;
	}
}
void pitchChange(s16 value)
{
	s16 newvalue = value;
	
	// Clamp to [-128, 127] 
	if(newvalue < -128) newvalue = -128; 
	if(newvalue > 127) newvalue = 127; 
	
	// Make positive
	newvalue += 128;
	
	// Scale to interval [0,2^14]
	u16 uvalue = newvalue * 64;
	
	// Split into lower and higher 7 bit
	u8 ls7b = uvalue & 0x7F;
	uvalue >>= 7;
	u8 ms7b = uvalue & 0x7F;
	
	dsmi_write(0xE0+channel, ls7b, ms7b);
}

void volChange(u8 value)
{
	u8 newvalue = value;
	
	// Clamp
	if(newvalue > 127) newvalue = 127;
	
	dsmi_write(0xB0+channel, 7, newvalue);
}

void printChannel()
{
	iprintf("\x1b[16;0H\x1b[KUsing MIDI channel %u\n", channel);
}

void printNote()
{
	iprintf("\x1b[17;0H\x1b[KLast Note: %u  \n", curnote);
}
void printMic()
{
	int i = 0;
	u8 temp = micavrg * 20 / 128;
	iprintf("\x1b[18;0H\x1b[KMic: %u  \n", micavrg);
	char vismic[22] = "                    |";
	for( ; i < temp; i++){
		vismic[i] = '+';
		iprintf("%c", vismic[i]);
	}
	for( ; i < 23; i++){
		vismic[i] = ' ';
		iprintf("%c", vismic[i]);
	}
	//iprintf("\n%u %u %u %u %u %u %u %u                \n%u %u    ", dmic[0],dmic[1],dmic[2],dmic[3],dmic[4],dmic[5],dmic[6],dmic[7],davrg,doldavrg);
}
void eraseBar(u8 x){
	//top screen
	u16 i;
	for(i = 0; i < 192; i++){
		bg3vram[x + (i * 256)] = RGB15(0,0,0);
		if (x>0) bg3vram[x + (i * 256) - 1] = RGB15(0,0,0);
		if (x<255) bg3vram[x + (i * 256) + 1] = RGB15(0,0,0);
	}
	//bottom screen
	for(i = 0; i < 192; i++){
		sub_bg3vram[x + (i * 256)] = RGB15(0,0,0);
		if (x>0) sub_bg3vram[x + (i * 256) - 1] = RGB15(0,0,0);
		if (x<255) sub_bg3vram[x + (i * 256) + 1] = RGB15(0,0,0);
	}
}
void drawBar(u8 r, u8 g, u8 b, u8 x, s16 bright){
	//top screen
	u16 i;
	s16 temp = bright;
	for(i = 192; i > 32; i--){
		bg3vram[x + (i * 256)] = RGB15(bright+r, bright+g,bright+b) | BIT(15);
		if (x>0) bg3vram[x + (i * 256) - 1] = RGB15(bright+r/2, bright+g/2, bright+b/2) | BIT(15);
		if (x<255) bg3vram[x + (i * 256) + 1] = RGB15(bright+r/2, bright+g/2, bright+b/2) | BIT(15);
	}
	for(i = 32; i > 0; i--){
		temp -= 1;
		bg3vram[x + (i * 256)] = RGB15(POS(temp+r), POS(temp+g), POS(temp+b)) | BIT(15);
		if (x>0) bg3vram[x + (i * 256) - 1] = RGB15(POS(temp+r/2), POS(temp+g/2), POS(temp+b/2)) | BIT(15);
		if (x<255) bg3vram[x + (i * 256) + 1] = RGB15(POS(temp+r/2), POS(temp+g/2), POS(temp+b/2)) | BIT(15);
	}
	//bottom screen
	temp = bright;
	for(i = 0; i < 160; i++){
		sub_bg3vram[x + (i * 256)] = RGB15(bright+r, bright+g,bright+b) | BIT(15);
		if (x>0) sub_bg3vram[x + (i * 256) - 1] = RGB15(bright+r/2, bright+g/2, bright+b/2) | BIT(15);
		if (x<255) sub_bg3vram[x + (i * 256) + 1] = RGB15(bright+r/2, bright+g/2, bright+b/2) | BIT(15);
	}
	for(i = 160; i < 192; i++){
		temp -= 1;
		sub_bg3vram[x + (i * 256)] = RGB15(POS(temp+r), POS(temp+g), POS(temp+b)) | BIT(15);
		if (x>0) sub_bg3vram[x + (i * 256) - 1] = RGB15(POS(temp+r/2), POS(temp+g/2), POS(temp+b/2)) | BIT(15);
		if (x<255) sub_bg3vram[x + (i * 256) + 1] = RGB15(POS(temp+r/2), POS(temp+g/2), POS(temp+b/2)) | BIT(15);
	}
}
void eraseNoteBar(){

}
void drawNoteBar(){

}
void update_GFX(){
	
	//s16 bright = (191 - y) * 128/192 ;
	//if(mic_enabled) bright = mic;
	//bright = bright >> 3;
	u8 i;
	for(i = 1; i < channel; i++){
		eraseBar(channelBars[i].x0);
		drawBar(6,12,9,channelBars[i].x,channelBars[i].bright);
	}
	for(i = channel + 1; i < 16; i++){
		eraseBar(channelBars[i].x0);
		drawBar(6,12,9,channelBars[i].x,channelBars[i].bright);
	}
	if(channel != 0){
		eraseBar(channelBars[channel].x0);
		drawBar(6,12,9,channelBars[i].x,channelBars[channel].bright);
	}
	for(i = 0; i<4; i++){
		eraseNoteBar();
		drawNoteBar();
	}	
}
void VblankHandler()
{
	scanKeys();
	touch = touchReadXY();
	u16 keys = keysDown();
	
	if(mic_enabled){	
		oldmicavrg = micavrg;
		//u8 oldmic = mic[micindx - 1]; //used for scaling
		u8 newmic;
		
		if (micindx > 7) micindx = 0;
		
		newmic = CommandGetMicVol(); //turns on mic, on first use...
		mic[micindx++] = newmic;//oldmic + ((newmic - oldmic) / 8); //scales difference...try 4
		micavrg = 0;
		
		for(int i = 0; i < 8; i++){
			micavrg += mic[i];
		}
		
		micavrg = oldmicavrg + (((micavrg / 8) - oldmicavrg) / 2);//scales difference of average...5
		printMic();
		
		wind_mic = micavrg;
		if(wind_mic > 127) wind_mic = 127;
		if(micavrg < 8){
			stop(lastnote);
			is_playing = 0;
		}
		if(!is_playing && micavrg > 7){
			play(curnote, wind_mic);
			lastnote = curnote;
			is_playing = 1;
		}
		volChange(wind_mic);
		channelBars[channel].h = wind_mic * 192/128;
		channelBars[channel].bright = wind_mic >> 3;
		//update_GFX(pm_x0, pm_y0, wind_mic);
		//if(micavrg > oldmicavrg) volChange(oldmicavrg + 5);
		//if(micavrg < oldmicavrg) volChange(oldmicavrg - 5);
	}	
	
	if(!touch_was_down && PEN_DOWN) { //new touch
		touch_was_down = 1;
		newTouch(touch.px, touch.py);
		printNote();
		if(keysHeld() & KEY_L){
			notes[noteindx] = curnote;
		}
		//update_GFX(touch.px, touch.py, wind_mic); //draws vertical bar where touched
		pm_x0 = touch.px;
		pm_y0 = touch.py;
	}
	else if(touch_was_down && !PEN_DOWN) //pen released
	{
		touch_was_down = 0;
		stop(lastnote);
		pitchChange(0);
	}

	if(touch_was_down && PEN_DOWN) { //dragging
		s16 dx;
		dx = touch.px - pm_x0;
		if(dx<0) dx = 0;
		u8 y = (191 - touch.py) * 128 / 192;
		if(y > 127) y = 127;
		
		pitchChange(dx);
		channelBars[channel].dx = dx;
		if(!mic_enabled) {
			volChange(y);
			channelBars[channel].h = 191 - touch.py;
			channelBars[channel].bright = y >> 3;
			//update_GFX(pm_x0, touch.py, 0);
		}
	}
	
	
	s16 tempnote = -1;
	if(keys & KEY_LEFT)
	{
		tempnote = notes[0];
		noteindx = 0;
	}
	if(keys & KEY_UP)
	{
		tempnote = notes[1];
		noteindx = 1;
	}
	if(keys & KEY_RIGHT)
	{
		tempnote = notes[2];
		noteindx = 2;
	}
	if(keys & KEY_DOWN)
	{
		tempnote = notes[3];
		noteindx = 3;
	}
	if(!(keysHeld() & KEY_L) && tempnote > -1){
		dsmi_write(0x90 | channel, tempnote, 100);
	}
	
	if(keys & KEY_START)
	{
		mic_enabled = 1; 
	}
	if(keys & KEY_SELECT)
	{
		mic_enabled = 0; //does not actually turn off mic
	}
	
	
	if(keys & KEY_X)
	{
		if(channel < 15)
			channel++;
		
		printChannel();
	}
	if(keys & KEY_B)
	{
		if(channel > 0)
			channel--;
		
		printChannel();
	}
	
	if(keys & KEY_Y)
	{
		dsmi_write(0x90 | channel, 71, micavrg);
	}
	if(keys & KEY_A)
	{
		stopAll(); 
	}
	
	/*if(keys & KEY_R){
		davrg = micavrg;
		doldavrg = oldmicavrg;
		for(int i = 0; i < 8; i++){
			dmic[i] = mic[i];
		}
	}*/
	update_GFX();
}

int main(void)
{
	powerON(POWER_ALL);
	irqInit();
	
	lcdMainOnTop();
	
	irqEnable(IRQ_VBLANK);

	// Set modes
	videoSetMode(MODE_5_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE);
	videoSetModeSub(MODE_5_2D | DISPLAY_BG3_ACTIVE);
	
	// Set banks
	vramSetMainBanks(VRAM_A_MAIN_BG_0x06000000, VRAM_B_MAIN_BG_0x06020000,
		VRAM_C_SUB_BG_0x06200000 , VRAM_D_LCD);
	
	// Gfx on sub
	SUB_BG3_CR = BG_BMP16_256x256 | BG_BMP_BASE(0);
	SUB_BG3_XDX = 1 << 8;
	SUB_BG3_XDY = 0;
	SUB_BG3_YDX = 0;
	SUB_BG3_YDY = 1 << 8;
	
	// Text bg on main
	BG0_CR = BG_MAP_BASE(4) | BG_TILE_BASE(0) | BG_PRIORITY(0);
	BG_PALETTE[255] = RGB15(24,0,0);//31 15 0
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK(4), (u16*)CHAR_BASE_BLOCK(0), 16);
	
	// bg on main
	BG2_CR = BG_BMP16_256x256 | BG_BMP_BASE(2) | BG_PRIORITY(1);
	BG2_XDX = 1 << 8;
	BG2_XDY = 0;
	BG2_YDX = 0;
	BG2_YDY = 1 << 8;
	
	//Gfx on main
	BG3_CR = BG_BMP16_256x256 | BG_BMP_BASE(8) | BG_PRIORITY(2);
	BG3_XDX = 1 << 8;
	BG3_XDY = 0;
	BG3_YDX = 0;
	BG3_YDY = 1 << 8;
	
	u16 i;
	for(i=0; i<256*64; ++i)
		bg2vram[i] = windidi_logo[i];
	
	channelBars[1].r = 2;
	
	noteBars[0].x = 60;
	noteBars[1].x = 64;
	noteBars[2].x = 68;
	noteBars[3].x = 70;
	//iprintf("\x1b[12;12HWindidi\n");
	
	dsmi_setup_wifi_support();
	
	iprintf("\x1b[10;0H\x1b[KConnecting\n");
	int res = dsmi_connect();
	
	if(res == 1) {
		iprintf("\x1b[10;0H\x1b[KOK\n");
	} else {
		iprintf("\x1b[10;0H\x1b[KOh no, could not connect!\n");
		while(1);
	}
	
	CommandInit();
	printChannel();
	
	iprintf("\x1b[12;0H X/B change MIDI channels!\n Start/Select MIC On/Off\n Hold L to assign notes to dpad!");
	
	while(1)
	{
		VblankHandler();

		swiWaitForVBlank();
	}
	
}
